from greenheart.simulation.technologies.offshore.fixed_platform import FixedPlatformDesign, FixedPlatformInstallation
from greenheart.simulation.technologies.offshore.floating_platform import FloatingPlatformDesign, FloatingPlatformInstallation
